import 'dart:io';

import 'package:args/command_runner.dart';
import 'package:nftgen/config.dart';
import 'package:nftgen/io.dart';
import 'package:nftgen/src/shared/projectmodel.dart';
import 'package:nftgen/streamprint.dart';
import 'package:path/path.dart';

class CidCommand extends Command {
  @override
  final name = "cid";
  @override
  final description = "Updates CID of generated metadata";

  CidCommand() {
    argParser
      ..addOption('project',
          abbr: "p",
          help: 'The project path',
          valueHelp: 'path',
          defaultsTo: Directory.current.absolute.path)
      ..addOption('cid',
          abbr: "c",
          help: 'Your CID',
          valueHelp: 'alphanumeric',
          mandatory: true);
  }

  @override
  void run() {
    final Directory projectDir = Directory(argResults!["project"]);
    final File projectFile = Io.getProject(projectDir);
    final ProjectModel projectJson = Io.readProjectJson(projectDir);
    final String cidReplace = argResults!["cid"];

    Io.checkFolderExists(projectJson.metaDir);

    Config.updateCidMetadata(projectFile, projectJson.metaDir,
        cidReplace: cidReplace, cidSearch: projectJson.cidCode);

    StreamPrint.prn("Updated CID: ${projectFile.path}");
    StreamPrint.prn(
        "Updated CID: ${normalize(projectJson.metaDir.path)}${Platform.pathSeparator}*.json");
  }
}
