import 'dart:io';

import 'package:args/command_runner.dart';
import 'package:nftgen/io.dart';
import 'package:nftgen/nft.dart';
import 'package:nftgen/src/shared/projectmodel.dart';
import 'package:nftgen/streamprint.dart';

class NftCommand extends Command {
  @override
  final name = "nft";
  @override
  final description = "Generates NFT images based on metadata";

  NftCommand() {
    argParser.addOption('project',
        abbr: "p",
        help: 'The project path',
        valueHelp: 'path',
        defaultsTo: Directory.current.absolute.path);
  }

  @override
  void run() async {
    Directory projectDir = Directory(argResults!["project"]);
    File projectFile =
        File('${projectDir.path}${Platform.pathSeparator}${Io.projectJson}');
    final ProjectModel projectJson = Io.readProjectJson(projectDir);
    Io.checkFolderExists(projectJson.metaDir);
    Io.checkFolderExists(projectJson.layerDir);

    await Nft.generateNft(projectFile, projectJson.layerDir,
        projectJson.imageDir, projectJson.metaDir);
    StreamPrint.prn("Created: ${projectFile.path}");
  }
}
