import 'dart:io';

import 'package:nftgen/io.dart';
import 'package:path/path.dart';

class ProjectLayerModel {
  String name;
  Directory directory;
  double probability;
  Map<String, int> weights;
  ProjectLayerModel(this.name, this.directory, this.probability, this.weights);

  Map<String, dynamic> toJson() {
    return {
      "name": name,
      "directory": directory.path,
      "probability": probability,
      "weights": weights
    };
  }

  static ProjectLayerModel fromJson(Map<String, dynamic> projectLayer) {
    return ProjectLayerModel(
        projectLayer["name"] as String,
        Directory(projectLayer["directory"]),
        projectLayer["probability"] as double,
        Map<String, int>.from(projectLayer["weights"]));
  }
}

class ProjectModel {
  final String name;
  final String cidCode;
  final int generateNfts;
  // final double weightsFactor;
  final List<ProjectLayerModel> layers;
  //
  final Directory metaDir;
  final Directory layerDir;
  final Directory imageDir;
  final Directory rarityDir;
  final File rarityNftCsv;
  final File rarityLayersCsv;
  final File rarityNftPng;
  final File rarityLayersPng;
  ProjectModel(
      this.name,
      this.cidCode,
      this.generateNfts,
      // this.weightsFactor,
      this.layers,
      //
      this.metaDir,
      this.layerDir,
      this.imageDir,
      this.rarityDir,
      this.rarityNftCsv,
      this.rarityLayersCsv,
      this.rarityNftPng,
      this.rarityLayersPng);

  static ProjectModel init(String name, int generateNfts, double factorMaxNft,
      List<ProjectLayerModel> projectLayers, Directory layersDir) {
    return ProjectModel(
      name,
      "<-- Your CID code -->",
      (generateNfts * factorMaxNft).toInt(),
      projectLayers,
      Directory("meta"),
      Directory(normalize(layersDir.absolute.path)),
      Directory("image"),
      Directory("rarity"),
      File("rarity_nft.csv"),
      File("rarity_layers.csv"),
      File("rarity_nft.png"),
      File("rarity_layers.png"),
    );
  }

  static ProjectModel fromFile(File projectFile) {
    final projectDir = projectFile.parent;
    Map<String, dynamic> projectJson = Io.readJson(projectFile);

    final List<Map<String, dynamic>> layersJson =
        List<Map<String, dynamic>>.from(projectJson['layers']);

    final List<ProjectLayerModel> layers = layersJson
        .map((layerJson) => ProjectLayerModel.fromJson(layerJson))
        .toList();

    return ProjectModel(
      projectJson['name'] as String,
      projectJson["cidCode"] as String,
      projectJson["generateNfts"] as int,
      // projectJson['weightsFactor'],
      // projectJson['layers'],
      layers,
      //
      Directory(projectDir.path + Io.sep + projectJson['metaDir']),
      Directory(projectJson['layerDir']), // NOT IN PROJECT!
      Directory(projectDir.path + Io.sep + projectJson['imageDir']),
      Directory(projectDir.path + Io.sep + projectJson['rarityDir']),
      File(projectDir.path +
          Io.sep +
          projectJson['rarityDir'] +
          Io.sep +
          projectJson["rarityNftCsv"]),
      File(projectDir.path +
          Io.sep +
          projectJson['rarityDir'] +
          Io.sep +
          projectJson["rarityLayersCsv"]),
      File(projectDir.path +
          Io.sep +
          projectJson['rarityDir'] +
          Io.sep +
          projectJson["rarityNftPng"]),
      File(projectDir.path +
          Io.sep +
          projectJson['rarityDir'] +
          Io.sep +
          projectJson["rarityLayersPng"]),
    );
  }

  addLayer(String name, Directory directory, double probability,
      Map<String, int> weights) {
    final layer = ProjectLayerModel(name, directory, probability, weights);
    layers.add(layer);
  }

  Map<String, dynamic> toJson() {
    final layersJson = [];
    for (var layer in layers) {
      layersJson.add(layer.toJson());
    }

    return {
      "name": name,
      "cidCode": cidCode,
      "generateNfts": generateNfts,
      // "weightsFactor": weightsFactor,
      "layers": layersJson,
      //
      "metaDir": metaDir.path,
      "layerDir": layerDir.path, // NOT IN PROJECT!
      "imageDir": imageDir.path,
      "rarityDir": rarityDir.path,
      "rarityNftCsv": rarityNftCsv.path,
      "rarityLayersCsv": rarityLayersCsv.path,
      "rarityNftPng": rarityNftPng.path,
      "rarityLayersPng": rarityLayersPng.path,
    };
  }
}
